#---------------------------------------------------------------
# Project         : LADS
# Module          : lads
# File            : ladsanomalies.py
# Version         : $Id: ladsanomalies.py,v 1.4 2003/02/15 22:26:07 fred Exp $
# Author          : Frederic Lepied
# Created On      : Mon Feb  3 08:27:58 2003
# Purpose         : Anomaly detection functions
#---------------------------------------------------------------

import socket
import string

from ladsconst import *
from ladsconfig import *

def get_host(h):
    if h == '':
        return 'local'
    try:
        return socket.gethostbyaddr(h)[0]
    except:
        return h

def is_exception(excp, user, host):
    ip = get_host(host)
    for e in excp:
        if e == host:
            return 1
        spl = string.split(e, '@')
        # match user and host
        if len(spl) == 2:
            if spl[0] == user:
                # exception is in the form of user@
                if spl[1] == '':
                    return 1
                # exception is in the form of user@regexp
                if re.search('^' + spl[1] + '$', host):
                    return 1
                # exception is in the form of user@host
                # and resolv host to allow one way resolution for dynamic
                # dns
                if ip == get_host(spl[1]):
                    return 1
        # match host
        if len(spl) == 1:
            # match as regexp
            if re.search('^' + spl[0] + '$', host):
                return 1
            # match as ip
            if get_host(host) == get_host(spl[0]):
                return 1
    return 0

def first_login_anomaly(entry, users, terms):
    if entry[UTMP_USER] not in users:
        return ('First login of %s' % entry[UTMP_USER], )
    else:
        return None

def multiple_login_anomaly(entry, users, terms):
    user = entry[UTMP_USER]
    excp = get_exception('foreign_login')

    if is_exception(excp, user, entry[UTMP_HOST]):
        return None
    
    for t in terms.values():
        if user != t[UTMP_USER]:
            continue
        
        if is_exception(excp, user, t[UTMP_HOST]):
            continue
        
        if entry[UTMP_HOST] != t[UTMP_HOST]:
            return ('Multiple login of %s' % user, 'Logged from %s(%s) and %s(%s)' % (entry[UTMP_HOST], get_host(entry[UTMP_HOST]), t[UTMP_HOST], get_host(t[UTMP_HOST])))
    return None

def foreign_login_anomaly(entry, users, terms):
    host = entry[UTMP_HOST]
    user = entry[UTMP_USER]

    if is_exception(get_exception('foreign_login'), user, host):
        print login, 'exception'
        return None

    return ('Foreign login of %s from %s' % (user, host), 'Host name is %s' % get_host(host))

# ladsanomalies.py ends here
